from typing import Callable, Optional

from discovery.client import Consul

from stfdigital.api import APPecas, IGIdentidades
from stfdigital.contexto import Contexto
from stfdigital.http.aiohttp import AIOHTTPEngine
from stfdigital.http.base import Engine


class API:
    def __init__(
        self,
        username: str,
        password: str,
        connector: Callable,
        discovery: Optional[Consul] = None,
        http: Engine = AIOHTTPEngine(),
    ):
        self.username = username
        self.password = password
        self.discovery = discovery
        if connector.__name__ == "cluster_connector" and not discovery:
            raise RuntimeError(
                "To use cluster_connector it's necessary specify a valid Consul instance."
            )
        self.connector = connector
        self._http = http

        self.ig_identidades = IGIdentidades(self)
        self.ap_pecas = APPecas(self)

    def __repr__(self) -> str:
        return f"API(username='{self.username}', password='{self.password}', connector='{self.connector.__name__}')"

    async def get(self, contexto: Contexto, path: str, *args, **kwargs):
        token_authorization_header = (
            await self.ig_identidades.token_authorization_header()
        )
        uri = await self.connector(self, contexto.value, path)
        response = await self._http.get(
            uri, *args, **kwargs, headers=token_authorization_header
        )
        return response

    async def post(self, contexto: Contexto, path: str, *args, **kwargs):
        uri = await self.connector(self, contexto.value, path)
        response = await self._http.post(uri, *args, **kwargs)
        return response


async def init_api(*args, **kwargs):
    return API(*args, **kwargs)
